<?php

namespace Drupal\ai_media_data_fetch\Form;

use Drupal\ai\Service\AiProviderFormHelper;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\TwigEnvironment;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure AI Translate module.
 */
class AiMediaDataFetchSettingsForm extends ConfigFormBase {

  use StringTranslationTrait;

  /**
   * Config settings.
   */
  const CONFIG_NAME = 'ai_media_data_fetch.settings';

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Twig engine.
   *
   * @var \Drupal\Core\Template\TwigEnvironment
   */
  protected TwigEnvironment $twig;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->twig = $container->get('twig');
    $instance->moduleHandler = $container->get('module_handler');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_media_data_fetch';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Load config.
    $config = $this->config(static::CONFIG_NAME);

    $form_helper = \Drupal::service('ai.form_helper');
    $form_helper->generateAiProvidersForm(
      $form,
      $form_state,
      'chat',
      '',
      AiProviderFormHelper::FORM_CONFIGURATION_FULL,
    );

    // @todo Currently i found no way to set the stored values as default values.
    $form["ai_provider"]["#default_value"] = $config->get('provider') ?? $form["ai_provider"]["#default_value"];
    $form["ajax_prefix"]["ai_model"]["#default_value"] = $config->get('model') ?? $form["ai_model"]["#default_value"];
    $form["ajax_prefix"]["ajax_prefix_configuration_max_tokens"]["#default_value"] = $config->get('configuration')['max_tokens'] ?? $form["ajax_prefix_configuration_max_tokens"]["#default_value"];
    $form["ajax_prefix"]["ajax_prefix_configuration_temperature"]["#default_value"] = $config->get('configuration')['temperature'] ?? $form["ajax_prefix_configuration_temperature"]["#default_value"];
    $form["ajax_prefix"]["ajax_prefix_configuration_frequency_penalty"]["#default_value"] = $config->get('configuration')['frequency_penalty'] ?? $form["ajax_prefix_configuration_frequency_penalty"]["#default_value"];
    $form["ajax_prefix"]["ajax_prefix_configuration_presence_penalty"]["#default_value"] = $config->get('configuration')['presence_penalty'] ?? $form["ajax_prefix_configuration_presence_penalty"]["#default_value"];
    $form["ajax_prefix"]["ajax_prefix_configuration_top_p"]["#default_value"] = $config->get('configuration')['top_p'] ?? $form["ajax_prefix_configuration_top_p"]["#default_value"];

    $form['system_role'] = [
      '#title' => $this->t('Default system prompt'),
      '#type' => 'textarea',
      '#default_value' => $config->get('system_role') ?? '',
      '#description' => $this->t('Prompt infos are needed here.'),
    ];

    $form['prompt'] = [
      '#title' => $this->t('Default generation prompt'),
      '#type' => 'textarea',
      '#default_value' => $config->get('prompt') ?? '',
      '#description' => $this->t('Prompt infos are needed here.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // @todo Pretty sure that one day we need to validate the prompt.

    $form_helper = \Drupal::service('ai.form_helper');
    return $form_helper->validateAiProvidersConfig($form, $form_state, 'chat', '');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_helper = \Drupal::service('ai.form_helper');

    $config = $form_helper->generateAiProvidersConfigurationFromForm(
      $form,
      $form_state,
      'chat',
      ''
    );

    $this->config(static::CONFIG_NAME)
      ->set('provider', $form_state->getValue("ai_provider"))
      ->set('model', $form_state->getValue("ai_model"))
      ->set('configuration', $config)
      ->set('system_role', $form_state->getValue('system_role'))
      ->set('prompt', $form_state->getValue('prompt'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
